from .augemtation_parameters import AugemntationParameter
import dm_pix as pix
from jax import numpy as jnp, random



def add_white_noise(maps: jnp.ndarray, mu: jnp.ndarray, sigma: jnp.ndarray, key: random.PRNGKey
                    ) -> tuple[jnp.ndarray, AugemntationParameter, random.PRNGKey]:
    key2use, key4next = random.split(key)
    function_kwargs = dict(key=key2use, shape=maps.shape, dtype=maps.dtype)
    input_kwargs = dict(mu=mu, sigma=sigma)
    maps = maps.at[:].add(random.normal(**function_kwargs) * sigma + mu)
    
    parameters = AugemntationParameter(
        add_gaussian_blur.__globals__['__name__'],
        add_gaussian_blur.__name__,
        function_kwargs,
        input_kwargs)
    
    return maps, parameters, key4next


def add_gaussian_blur(maps: jnp.ndarray, sigma: jnp.ndarray, kernel_size: jnp.ndarray, key: random.PRNGKey
                      ) -> tuple[jnp.ndarray, AugemntationParameter, random.PRNGKey]:
    key4next = key
    function_kwargs = dict(sigma=sigma, kernel_size=kernel_size)
    input_kwargs = dict(kernel_size=kernel_size, sigma=sigma)
    maps = pix.gaussian_blur(maps, **function_kwargs)

    parameters = AugemntationParameter(
        add_gaussian_blur.__globals__['__name__'],
        add_gaussian_blur.__name__,
        function_kwargs,
        input_kwargs)
    
    return maps, parameters, key4next
