from matplotlib import pyplot as plt
from jax import numpy as jnp
from pathlib import Path


def save_map_plot_with_index(maps: jnp.ndarray, index: int, save_path: Path, figsize=(8, 8), 
                             title: str = None): 
    fig = plt.figure(figsize=figsize)
    ax = fig.add_subplot(111)
    ax.imshow(maps[:, :, index])
    ax.set_xlabel(r'$z$ / mm')
    ax.set_ylabel(r'$\alpha$ / rad')
    ax.set_xticks([0, int(maps.shape[0] // 2), int(maps.shape[0])]) 
    ax.set_xticklabels([r'$-3141.5$', r'$0.$',  r'$3141.5$']) 
    ax.set_yticks([0, int(maps.shape[1] // 2), int(maps.shape[1])]) 
    ax.set_yticklabels([r'$-\pi$', r'$0.$',  r'$\pi$'])    

    if title is not None:
        ax.set_title(title)

    plt.tight_layout()

    plt.savefig(save_path)
    plt.close(fig)
    